//---------------------------------------------------------------------------
// TestBoleVolumeCalculator
//---------------------------------------------------------------------------
#if !defined(TestBoleVolumeCalculator_H)
  #define TestBoleVolumeCalculator_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the class clBoleVolumeCalculator.
*/
class clTestBoleVolumeCalculator : public CppUnit::TestFixture {

  public:

  /**
  * Performs all the tests from one function.
  */
  void TestBoleVolumeCalculator();

  protected:

  /**
  * Tests normal processing.  WriteXMLFile1() does setup.  Then 87 trees are
  * created.  A simulation is run for one timestep then the volume checked
  * for all those trees.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessing(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  This uses WriteXMLErrorFile1(),
  * WriteXMLErrorFile2(), and WriteXMLErrorFile3() to introduce errors and
  * make sure that they are caught.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file for normal processing.  There are 8 species (of
  * which 7 are used), and the bole volume behavior is the only one in the
  * run.
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes an error file where a form class value is less than 60.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error file where a form class value is greater than 100.
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes an error file where the volume behavior is applied to seedlings.
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * Ends with &lt;/trees&gt;.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);


};
//---------------------------------------------------------------------------
#endif // TestBoleVolumeCalculator_H
