//---------------------------------------------------------------------------
// TestEstablishment
//---------------------------------------------------------------------------
#if !defined(TestEstablishment_H)
  #define TestEstablishment_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* This class tests the clEstablishment behavior.
 */
class clTestEstablishment : public CppUnit::TestFixture {

  public:

  /**
  * Groups all establishment tests into one function.
  */
  void TestEstablishment();


  private:

  /**
  * Tests establishment.  WriteXMLFile1() does setup.  Non-spatial dispserse is
  * included with parameters such that it creates no seeds.  The "Dispersed
  * Seeds" grid cells do not divide evenly into the plot lengths.  The grid is
  * populated with some initial values.  Then one timestep of a simulation is
  * run and this verifies that the number of seedlings is right, and the number
  * of seeds is 0.
  * @param p_oSimManager Sim Manager object to run testing.
  */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);

  /**
  * Tests establishment.  WriteXMLFile2() does setup.  Non-spatial dispserse is
  * included with parameters such that it creates no seeds.  The "Dispersed
  * Seeds" grid cells divide evenly into the plot lengths.  The grid is
  * populated with some initial values.  Then one timestep of a simulation is
  * run and this verifies that the number of seedlings is right, and the
  * number of seeds is 0.
  * @param p_oSimManager Sim Manager object to run testing.
  */
  void TestNormalProcessingRun2(clSimManager *p_oSimManager);

  /**
  * Runs through the error files to make sure that errors are caught correctly.
  * @param p_oSimManager Sim Manager object to run testing.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file with no disperse behaviors whatsoever.
  * @return Filename written.
  */
  char * WriteXMLErrorFile1();
};

//---------------------------------------------------------------------------
#endif // TestEstablishment
