//---------------------------------------------------------------------------
// TestLognormalGrowth.cpp
//---------------------------------------------------------------------------
#include "TestLognormalGrowth.h"
#include <fstream>
#include <math.h>
#include "TreePopulation.h"
#include "SimManager.h"
#include "BehaviorBase.h"

////////////////////////////////////////////////////////////////////////////
//  TestLognormalGrowth()
////////////////////////////////////////////////////////////////////////////
void clTestLognormalGrowth::TestLognormalGrowth() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  Test1YearPerTimestep(p_oSimManager);
  Test3YearsPerTimestep(p_oSimManager);

  delete p_oSimManager;
}

////////////////////////////////////////////////////////////////////////////
//  Test1YearPerTimestep()
////////////////////////////////////////////////////////////////////////////
void clTestLognormalGrowth::Test1YearPerTimestep(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTree1, *p_oTree2, *p_oTree3, *p_oTree4, *p_oTree5, *p_oTree6,
     *p_oTree7, *p_oTree8;
  float fDiam, fHeight, fExpectedDiam, fExpectedHeight, fLight, fGrowth, fExpectedGrowth;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our eight trees and set diameter, height, and GLI for each
  //Seedling, Species 1
  fDiam = 0.7;
  fHeight = 0.8;
  fLight = 10;
  p_oTree1 = p_oPop->CreateTree(1, 1, 0, clTreePopulation::seedling, fDiam);
  p_oTree1->SetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), fHeight, false, false);
  p_oTree1->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree1->GetSpecies(), p_oTree1->GetType()), fLight, false, false);

  //Seedling, Species 1
  fDiam = 0.1;
  fHeight = 0.2;
  fLight = 90;
  p_oTree2 = p_oPop->CreateTree(2, 2, 0, clTreePopulation::seedling, fDiam);
  p_oTree2->SetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), fHeight, false, false);
  p_oTree2->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree2->GetSpecies(), p_oTree2->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 3.2032;
  fHeight = 5;
  fLight = 12;
  p_oTree3 = p_oPop->CreateTree(3, 3, 0, clTreePopulation::sapling, fDiam);
  p_oTree3->SetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), fHeight, false, false);
  p_oTree3->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree3->GetSpecies(), p_oTree3->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 6.4064;
  fHeight = 3;
  fLight = 100;
  p_oTree4 = p_oPop->CreateTree(4, 4, 0, clTreePopulation::sapling, fDiam);
  p_oTree4->SetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), fHeight, false, false);
  p_oTree4->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree4->GetSpecies(), p_oTree4->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 0.2;
  fHeight = 0.8;
  fLight = 0;
  p_oTree5 = p_oPop->CreateTree(5, 5, 1, clTreePopulation::seedling, fDiam);
  p_oTree5->SetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), fHeight, false, false);
  p_oTree5->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree5->GetSpecies(), p_oTree5->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 1.4;
  fHeight = 0.65;
  fLight = 95;
  p_oTree6 = p_oPop->CreateTree(6, 6, 1, clTreePopulation::seedling, fDiam);
  p_oTree6->SetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), fHeight, false, false);
  p_oTree6->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree6->GetSpecies(), p_oTree6->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 1.7832;
  fHeight = 4;
  fLight = 7;
  p_oTree7 = p_oPop->CreateTree(7, 7, 1, clTreePopulation::sapling, fDiam);
  p_oTree7->SetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), fHeight, false, false);
  p_oTree7->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree7->GetSpecies(), p_oTree7->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 4.1608;
  fHeight = 6.3;
  fLight = 100;
  p_oTree8 = p_oPop->CreateTree(8, 8, 1, clTreePopulation::sapling, fDiam);
  p_oTree8->SetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), fHeight, false, false);
  p_oTree8->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree8->GetSpecies(), p_oTree8->GetType()), fLight, false, false);

  //Now run growth, but skip light
  int i;
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  //Confirm all trees
  //Seedling, Species 1
  fExpectedDiam = 0.780228614;
  fExpectedHeight = 0.880211446;
  fExpectedGrowth = 0.40114307;
  p_oTree1->GetValue(p_oPop->GetDiam10Code(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fDiam);
  p_oTree1->GetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fHeight);
  p_oTree1->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree1->GetSpecies(), p_oTree1->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 1
  fExpectedDiam = 0.102039497;
  fExpectedHeight = 0.202084091;
  fExpectedGrowth = 0.010197483;
  p_oTree2->GetValue(p_oPop->GetDiam10Code(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fDiam);
  p_oTree2->GetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fHeight);
  p_oTree2->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree2->GetSpecies(), p_oTree2->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 4.26;
  fExpectedHeight = 6.693804933;
  fExpectedGrowth = 1.3;
  p_oTree3->GetValue(p_oPop->GetDiam10Code(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fDiam);
  p_oTree3->GetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fHeight);
  p_oTree3->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree3->GetSpecies(), p_oTree3->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 8.26;
  fExpectedHeight = 5.631045457;
  fExpectedGrowth =  1.3;
  p_oTree4->GetValue(p_oPop->GetDiam10Code(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fDiam);
  p_oTree4->GetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fHeight);
  p_oTree4->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree4->GetSpecies(), p_oTree4->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2
  fExpectedDiam = 0.2;
  fExpectedHeight = 0.8;
  fExpectedGrowth = 0;
  p_oTree5->GetValue(p_oPop->GetDiam10Code(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fDiam);
  p_oTree5->GetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fHeight);
  p_oTree5->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree5->GetSpecies(), p_oTree5->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2
  fExpectedDiam = 2.082918043;
  fExpectedHeight = 1.160589557;
  fExpectedGrowth = 3.414590215;
  p_oTree6->GetValue(p_oPop->GetDiam10Code(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fDiam);
  p_oTree6->GetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fHeight);
  p_oTree6->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree6->GetSpecies(), p_oTree6->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 3.16025302;
  fExpectedHeight = 3.040339823;
  fExpectedGrowth = 0.801265098;
  p_oTree7->GetValue(p_oPop->GetDiam10Code(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fDiam);
  p_oTree7->GetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fHeight);
  p_oTree7->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree7->GetSpecies(), p_oTree7->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 10.59661622;
  fExpectedHeight = 6.728575917;
  fExpectedGrowth = 17.98308112;
  p_oTree8->GetValue(p_oPop->GetDiam10Code(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fDiam);
  p_oTree8->GetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fHeight);
  p_oTree8->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree8->GetSpecies(), p_oTree8->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);
}

////////////////////////////////////////////////////////////////////////////
//  Test3YearsPerTimestep()
////////////////////////////////////////////////////////////////////////////
void clTestLognormalGrowth::Test3YearsPerTimestep(clSimManager *p_oSimManager) {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTree *p_oTree1, *p_oTree2, *p_oTree3, *p_oTree4, *p_oTree5, *p_oTree6,
     *p_oTree7, *p_oTree8;
  float fDiam, fHeight, fExpectedDiam, fExpectedHeight, fLight, fGrowth, fExpectedGrowth;

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile2() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our eight trees and set diameter, height, and GLI for each
  //Seedling, Species 1
  fDiam = 0.7;
  fHeight = 0.8;
  fLight = 10;
  p_oTree1 = p_oPop->CreateTree(1, 1, 0, clTreePopulation::seedling, fDiam);
  p_oTree1->SetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), fHeight, false, false);
  p_oTree1->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree1->GetSpecies(), p_oTree1->GetType()), fLight, false, false);

  //Seedling, Species 1
  fDiam = 0.1;
  fHeight = 0.2;
  fLight = 90;
  p_oTree2 = p_oPop->CreateTree(2, 2, 0, clTreePopulation::seedling, fDiam);
  p_oTree2->SetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), fHeight, false, false);
  p_oTree2->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree2->GetSpecies(), p_oTree2->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 3.2032;
  fHeight = 5;
  fLight = 12;
  p_oTree3 = p_oPop->CreateTree(3, 3, 0, clTreePopulation::sapling, fDiam);
  p_oTree3->SetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), fHeight, false, false);
  p_oTree3->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree3->GetSpecies(), p_oTree3->GetType()), fLight, false, false);

  //Sapling, Species 1
  fDiam = 6.4064;
  fHeight = 3;
  fLight = 100;
  p_oTree4 = p_oPop->CreateTree(4, 4, 0, clTreePopulation::sapling, fDiam);
  p_oTree4->SetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), fHeight, false, false);
  p_oTree4->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree4->GetSpecies(), p_oTree4->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 0.2;
  fHeight = 0.8;
  fLight = 0;
  p_oTree5 = p_oPop->CreateTree(5, 5, 1, clTreePopulation::seedling, fDiam);
  p_oTree5->SetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), fHeight, false, false);
  p_oTree5->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree5->GetSpecies(), p_oTree5->GetType()), fLight, false, false);

  //Seedling, Species 2
  fDiam = 1.4;
  fHeight = 0.65;
  fLight = 95;
  p_oTree6 = p_oPop->CreateTree(6, 6, 1, clTreePopulation::seedling, fDiam);
  p_oTree6->SetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), fHeight, false, false);
  p_oTree6->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree6->GetSpecies(), p_oTree6->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 1.7832;
  fHeight = 4;
  fLight = 7;
  p_oTree7 = p_oPop->CreateTree(7, 7, 1, clTreePopulation::sapling, fDiam);
  p_oTree7->SetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), fHeight, false, false);
  p_oTree7->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree7->GetSpecies(), p_oTree7->GetType()), fLight, false, false);

  //Sapling, Species 2
  fDiam = 4.1608;
  fHeight = 6.3;
  fLight = 100;
  p_oTree8 = p_oPop->CreateTree(8, 8, 1, clTreePopulation::sapling, fDiam);
  p_oTree8->SetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), fHeight, false, false);
  p_oTree8->SetValue(p_oPop->GetFloatDataCode("Light", p_oTree8->GetSpecies(), p_oTree8->GetType()), fLight, false, false);

  //Now run growth, but skip light
  int i;
  for (i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    clBehaviorBase *p_oBase = p_oSimManager->GetBehaviorObject(i);
    p_oBase->Action();
  }

  //Confirm all trees
  //Seedling, Species 1
  fExpectedDiam = 0.998505788;
  fExpectedHeight = 1.097331886;
  fExpectedGrowth = 0.497509646;
  p_oTree1->GetValue(p_oPop->GetDiam10Code(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fDiam);
  p_oTree1->GetValue(p_oPop->GetHeightCode(p_oTree1->GetSpecies(), p_oTree1->GetType()), &fHeight);
  p_oTree1->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree1->GetSpecies(), p_oTree1->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 1
  fExpectedDiam = 0.10647943;
  fExpectedHeight = 0.206620603;
  fExpectedGrowth = 0.010799049;
  p_oTree2->GetValue(p_oPop->GetDiam10Code(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fDiam);
  p_oTree2->GetValue(p_oPop->GetHeightCode(p_oTree2->GetSpecies(), p_oTree2->GetType()), &fHeight);
  p_oTree2->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree2->GetSpecies(), p_oTree2->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 4.78;
  fExpectedHeight = 10.20363193;
  fExpectedGrowth = 1.3;
  p_oTree3->GetValue(p_oPop->GetDiam10Code(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fDiam);
  p_oTree3->GetValue(p_oPop->GetHeightCode(p_oTree3->GetSpecies(), p_oTree3->GetType()), &fHeight);
  p_oTree3->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree3->GetSpecies(), p_oTree3->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 1
  fExpectedDiam = 8.78;
  fExpectedHeight = 10.95932427;
  fExpectedGrowth = 1.3;
  p_oTree4->GetValue(p_oPop->GetDiam10Code(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fDiam);
  p_oTree4->GetValue(p_oPop->GetHeightCode(p_oTree4->GetSpecies(), p_oTree4->GetType()), &fHeight);
  p_oTree4->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree4->GetSpecies(), p_oTree4->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2
  fExpectedDiam = 0.2;
  fExpectedHeight = 0.8;
  fExpectedGrowth = 0;
  p_oTree5->GetValue(p_oPop->GetDiam10Code(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fDiam);
  p_oTree5->GetValue(p_oPop->GetHeightCode(p_oTree5->GetSpecies(), p_oTree5->GetType()), &fHeight);
  p_oTree5->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree5->GetSpecies(), p_oTree5->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Seedling, Species 2
  fExpectedDiam = 4.997903769;
  fExpectedHeight = 3.137986528;
  fExpectedGrowth = 5.996506282;
  p_oTree6->GetValue(p_oPop->GetDiam10Code(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fDiam);
  p_oTree6->GetValue(p_oPop->GetHeightCode(p_oTree6->GetSpecies(), p_oTree6->GetType()), &fHeight);
  p_oTree6->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree6->GetSpecies(), p_oTree6->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 3.506987865;
  fExpectedHeight = 3.221183793;
  fExpectedGrowth = 0.844979776;
  p_oTree7->GetValue(p_oPop->GetDiam10Code(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fDiam);
  p_oTree7->GetValue(p_oPop->GetHeightCode(p_oTree7->GetSpecies(), p_oTree7->GetType()), &fHeight);
  p_oTree7->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree7->GetSpecies(), p_oTree7->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);

  //Sapling, Species 2
  fExpectedDiam = 12.20898764;
  fExpectedHeight = 11.08462419;
//  fExpectedGrowth = 22.56669929; No lgm - adult now
  p_oTree8->GetValue(p_oPop->GetDbhCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fDiam);
  p_oTree8->GetValue(p_oPop->GetHeightCode(p_oTree8->GetSpecies(), p_oTree8->GetType()), &fHeight);
//  p_oTree8->GetValue(p_oPop->GetFloatDataCode("Growth", p_oTree8->GetSpecies(), p_oTree8->GetType()), &fGrowth);
  CPPUNIT_ASSERT(fabs(fExpectedDiam - fDiam) < 0.001);
  CPPUNIT_ASSERT(fabs(fExpectedHeight - fHeight) < 0.001);
//  CPPUNIT_ASSERT(fabs(fExpectedGrowth - fGrowth) < 0.001);
}


////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestLognormalGrowth::WriteXMLFile1() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>"
       << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       //Leave in one light so that lgm will be registerd
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>lognormal growth diam only</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>lognormal growth height only</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>constradialgrowth diam only</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>height incrementer</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>lognormal growth</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "<growth>"
       << "<gr_adultConstRadialInc>"
       << "<gr_acriVal species=\"Species_1\">1.3</gr_acriVal>"
       << "<gr_acriVal species=\"Species_2\">5.92</gr_acriVal>"
       << "</gr_adultConstRadialInc>"
       << "<gr_diamLognormalIncAtDiam36>"
       << "<gr_dliad36Val species=\"Species_1\">35.88</gr_dliad36Val>"
       << "<gr_dliad36Val species=\"Species_2\">31.2</gr_dliad36Val>"
       << "</gr_diamLognormalIncAtDiam36>"
       << "<gr_heightLognormalIncAtDiam36>"
       << "<gr_hliad36Val species=\"Species_1\">321.8</gr_hliad36Val>"
       << "<gr_hliad36Val species=\"Species_2\">241.08</gr_hliad36Val>"
       << "</gr_heightLognormalIncAtDiam36>"
       << "<gr_diamLognormalShapeParam>"
       << "<gr_dlspVal species=\"Species_1\">1.46</gr_dlspVal>"
       << "<gr_dlspVal species=\"Species_2\">1.56</gr_dlspVal>"
       << "</gr_diamLognormalShapeParam>"
       << "<gr_heightLognormalShapeParam>"
       << "<gr_hlspVal species=\"Species_1\">2.37</gr_hlspVal>"
       << "<gr_hlspVal species=\"Species_2\">1.87</gr_hlspVal>"
       << "</gr_heightLognormalShapeParam>"
       << "<gr_diamLognormalEffectOfShade>"
       << "<gr_dleosVal species=\"Species_1\">0.37</gr_dleosVal>"
       << "<gr_dleosVal species=\"Species_2\">0.9</gr_dleosVal>"
       << "</gr_diamLognormalEffectOfShade>"
       << "<gr_heightLognormalEffectOfShade>"
       << "<gr_hleosVal species=\"Species_1\">0.1</gr_hleosVal>"
       << "<gr_hleosVal species=\"Species_2\">0.87</gr_hleosVal>"
       << "</gr_heightLognormalEffectOfShade>"
       << "</growth>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestLognormalGrowth::WriteXMLFile2() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
         << "<paramFile fileCode=\"06010101\">"
         << "<plot>"
         << "<timesteps>1</timesteps>"
         << "<yearsPerTimestep>3</yearsPerTimestep>"
         << "<randomSeed>1</randomSeed>"
         << "<plot_lenX>200.0</plot_lenX>"
         << "<plot_lenY>200.0</plot_lenY>"
         << "<plot_latitude>55.37</plot_latitude>"
         << "</plot>"
         << "<trees>"
         << "<tr_speciesList>"
         << "<tr_species speciesName=\"Species_1\" />"
         << "<tr_species speciesName=\"Species_2\" />"
         << "</tr_speciesList>"
         << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
         << "<tr_minAdultDBH>"
         << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
         << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
         << "</tr_minAdultDBH>"
         << "<tr_maxSeedlingHeight>"
         << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
         << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
         << "</tr_maxSeedlingHeight>"
         << "</trees>"
         << "<allometry>"
         << "<tr_canopyHeight>"
         << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
         << "<tr_chVal species=\"Species_2\">39.54</tr_chVal>"
         << "</tr_canopyHeight>"
         << "<tr_stdAsympCrownRad>"
         << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
         << "<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>"
         << "</tr_stdAsympCrownRad>"
         << "<tr_stdCrownRadExp>"
         << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
         << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
         << "</tr_stdCrownRadExp>"
         << "<tr_conversionDiam10ToDBH>"
         << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
         << "<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>"
         << "</tr_conversionDiam10ToDBH>"
         << "<tr_interceptDiam10ToDBH>"
         << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
         << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
         << "</tr_interceptDiam10ToDBH>"
         << "<tr_stdAsympCrownHt>"
         << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
         << "<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>"
         << "</tr_stdAsympCrownHt>"
         << "<tr_stdCrownHtExp>"
         << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
         << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
         << "</tr_stdCrownHtExp>"
         << "<tr_slopeOfHeight-Diam10>"
         << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
         << "<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>"
         << "</tr_slopeOfHeight-Diam10>"
         << "<tr_slopeOfAsymHeight>"
         << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
         << "<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>"
         << "</tr_slopeOfAsymHeight>"
         << "<tr_whatSeedlingHeightDiam>"
         << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
         << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
         << "</tr_whatSeedlingHeightDiam>"
         << "<tr_whatSaplingHeightDiam>"
         << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
         << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
         << "</tr_whatSaplingHeightDiam>"
         << "<tr_whatAdultHeightDiam>"
         << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
         << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
         << "</tr_whatAdultHeightDiam>"
         << "<tr_whatAdultCrownRadDiam>"
         << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
         << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
         << "</tr_whatAdultCrownRadDiam>"
         << "<tr_whatAdultCrownHeightHeight>"
         << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
         << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
         << "</tr_whatAdultCrownHeightHeight>"
         << "<tr_whatSaplingCrownRadDiam>"
         << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
         << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
         << "</tr_whatSaplingCrownRadDiam>"
         << "<tr_whatSaplingCrownHeightHeight>"
         << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
         << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
         << "</tr_whatSaplingCrownHeightHeight>"
         << "</allometry>"
         << "<behaviorList>"
         << "<behavior>"
         //Leave in one light so that lgm will be registerd
         << "<behaviorName>radiallight</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
         << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
         << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
         << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>lognormal growth diam only</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
         << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>lognormal growth height only</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
         << "<applyTo species=\"Species_2\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>constradialgrowth diam only</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>height incrementer</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
         << "</behavior>"
         << "<behavior>"
         << "<behaviorName>lognormal growth</behaviorName>"
         << "<version>1</version>"
         << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
         << "</behavior>"
         << "</behaviorList>"
         << "<growth>"
         << "<gr_adultConstRadialInc>"
         << "<gr_acriVal species=\"Species_1\">1.3</gr_acriVal>"
         << "<gr_acriVal species=\"Species_2\">5.92</gr_acriVal>"
         << "</gr_adultConstRadialInc>"
         << "<gr_diamLognormalIncAtDiam36>"
         << "<gr_dliad36Val species=\"Species_1\">35.88</gr_dliad36Val>"
         << "<gr_dliad36Val species=\"Species_2\">31.2</gr_dliad36Val>"
         << "</gr_diamLognormalIncAtDiam36>"
         << "<gr_heightLognormalIncAtDiam36>"
         << "<gr_hliad36Val species=\"Species_1\">321.8</gr_hliad36Val>"
         << "<gr_hliad36Val species=\"Species_2\">241.08</gr_hliad36Val>"
         << "</gr_heightLognormalIncAtDiam36>"
         << "<gr_diamLognormalShapeParam>"
         << "<gr_dlspVal species=\"Species_1\">1.46</gr_dlspVal>"
         << "<gr_dlspVal species=\"Species_2\">1.56</gr_dlspVal>"
         << "</gr_diamLognormalShapeParam>"
         << "<gr_heightLognormalShapeParam>"
         << "<gr_hlspVal species=\"Species_1\">2.37</gr_hlspVal>"
         << "<gr_hlspVal species=\"Species_2\">1.87</gr_hlspVal>"
         << "</gr_heightLognormalShapeParam>"
         << "<gr_diamLognormalEffectOfShade>"
         << "<gr_dleosVal species=\"Species_1\">0.37</gr_dleosVal>"
         << "<gr_dleosVal species=\"Species_2\">0.9</gr_dleosVal>"
         << "</gr_diamLognormalEffectOfShade>"
         << "<gr_heightLognormalEffectOfShade>"
         << "<gr_hleosVal species=\"Species_1\">0.1</gr_hleosVal>"
         << "<gr_hleosVal species=\"Species_2\">0.87</gr_hleosVal>"
         << "</gr_heightLognormalEffectOfShade>"
         << "</growth>"
         << "</paramFile>";

  oOut.close();

  return cFileString;
}
